@@ -22,7 +22,6 @@ from __future__ import division, print_function, absolute_import
 
 import os
 import ast
-import sys
 import zlib
 import base64
 import random as rnd_module
@@ -49,12 +48,12 @@ from hypothesis._settings import local_settings, note_deprecation
 from hypothesis.executors import new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.statistics import note_engine_for_statistics
-from hypothesis.internal.compat import ceil, hbytes, qualname, \
+from hypothesis.internal.compat import PY2, ceil, hbytes, qualname, \
     binary_type, str_to_bytes, benchmark_time, get_type_hints, \
     getfullargspec, int_from_bytes, bad_django_TestCase
 from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.utils.conventions import infer, not_set
-from hypothesis.internal.escalation import \
+from hypothesis.internal.escalation import get_trimmed_traceback, \
     escalate_hypothesis_internal_error
 from hypothesis.internal.reflection import is_mock, proxies, nicerepr, \
     arg_string, impersonate, function_digest, fully_qualified_name, \
@@ -561,7 +560,6 @@ class StateForActualGivenExecution(object):
                     'Tests run under @given should return None, but '
                     '%s returned %r instead.'
                 ) % (self.test.__name__, result), HealthCheck.return_value)
-            return False
         except UnsatisfiedAssumption:
             data.mark_invalid()
         except (
@@ -571,16 +569,17 @@ class StateForActualGivenExecution(object):
             raise
         except Exception as e:
             escalate_hypothesis_internal_error()
-            data.__expected_traceback = traceback.format_exc()
+            tb = get_trimmed_traceback()
+            data.__expected_traceback = ''.join(
+                traceback.format_exception(type(e), e, tb)
+            )
             data.__expected_exception = e
             verbose_report(data.__expected_traceback)
 
-            error_class, _, tb = sys.exc_info()
-
             origin = traceback.extract_tb(tb)[-1]
             filename = origin[0]
             lineno = origin[1]
-            data.mark_interesting((error_class, filename, lineno))
+            data.mark_interesting((type(e), filename, lineno))
 
     def run(self):
         # Tell pytest to omit the body of this function from tracebacks
@@ -672,10 +671,11 @@ class StateForActualGivenExecution(object):
                     'Unreliable assumption: An example which satisfied '
                     'assumptions on the first run now fails it.'
                 )
-            except BaseException:
+            except BaseException as e:
                 if len(self.falsifying_examples) <= 1:
                     raise
-                report(traceback.format_exc())
+                tb = get_trimmed_traceback()
+                report(''.join(traceback.format_exception(type(e), e, tb)))
             finally:  # pragma: no cover
                 # This section is in fact entirely covered by the tests in
                 # test_reproduce_failure, but it seems to trigger a lovely set
@@ -922,11 +922,11 @@ def given(
                         setattr(runner, 'subTest', subTest)
                 else:
                     state.run()
-            except BaseException:
+            except BaseException as e:
                 generated_seed = \
                     wrapped_test._hypothesis_internal_use_generated_seed
-                if generated_seed is not None and not state.failed_normally:
-                    with local_settings(settings):
+                with local_settings(settings):
+                    if not (state.failed_normally or generated_seed is None):
                         if running_under_pytest:
                             report(
                                 'You can add @seed(%(seed)d) to this test or '
@@ -937,7 +937,25 @@ def given(
                             report(
                                 'You can add @seed(%d) to this test to '
                                 'reproduce this failure.' % (generated_seed,))
-                raise
+                    # The dance here is to avoid showing users long tracebacks
+                    # full of Hypothesis internals they don't care about.
+                    # We have to do this inline, to avoid adding another
+                    # internal stack frame just when we've removed the rest.
+                    if PY2:
+                        # Python 2 doesn't have Exception.with_traceback(...);
+                        # instead it has a three-argument form of the `raise`
+                        # statement.  Which is a SyntaxError on Python 3.
+                        exec(
+                            'raise type(e), e, get_trimmed_traceback()',
+                            globals(), locals()
+                        )
+                    # On Python 3, we swap out the real traceback for our
+                    # trimmed version.  Using a variable ensures that the line
+                    # which will actually appear in trackbacks is as clear as
+                    # possible - "raise the_error_hypothesis_found".
+                    the_error_hypothesis_found = \
+                        e.with_traceback(get_trimmed_traceback())
+                    raise the_error_hypothesis_found
 
         for attrib in dir(test):
             if not (attrib.startswith('_') or hasattr(wrapped_test, attrib)):
